// *****************************************************************************
// Name:        Kyro Control Panel (Windows only)
// Purpose:     Change registry values and keys
// Copyright:   (C) 2005, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>
// *****************************************************************************

/**
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <wx/wx.h>
#include <wx/notebook.h>
#include <wx/file.h>
#include <wx/image.h>
#include <wx/msw/registry.h>
#include <wx/html/htmlwin.h>

// **************************
// Constants
// **************************

static const wxString name("Kyro Control Panel");       // prog name
static const wxString version("05.04.20");              // prog version
static const wxString datum("20. April 2005");          // date of built

static wxArrayString RegSet;                            // RegSettings strings

// **************************
// Event-IDs
// **************************

enum {
 // Direct3D registry settings (order relevant)
  ID_FORCE_LARGE_TEX_TO_DXT1=0,             // activate texture compression (standard 0)
  ID_USE_RHW,                             // use W-Buffering (standard 0)
  ID_ZLOAD,                               // load depth/stencil-buffer (standard 0)
  ID_ZSTORE,                              // store depth/stencil-buffer (standard 0)
  ID_ZTRANSLATE,

  ID_ENABLE_SCENESS,                      // activate antialias (standard 0)
  ID_DISABLE_XSS,                         // deactivate hor. antialias (standard 1)
  ID_DISABLE_YSS,                         // deavtivate vert. antialias (standard 1)
  ID_ENABLE_SOFTPT,                       // use smooth edges (standard 0)
  ID_ENABLE_TEXTURESS,                    // activate anisotrop filtering (standard 0)
  ID_DISABLE_ANISO,  
  ID_ENABLE_TEXTURE_TLF,                  // activate trilinear filtering (standard 0)
  ID_DISABLE_FAST_TLF,    
  ID_DONT_DISABLE_FAST_TLF,

  ID_DISABLE_ENTNL,                           // deactivate Enhanced TnL (standard 1) 
  ID_DISABLE_MID_SCENE,    
  ID_DXT_EXPOSE_CAPS,
  ID_EXPOSE_CUBEMAP_CAPS,
  ID_EXPOSE_2KTEXTURES,
  ID_NORENDERS_TILL_FLIP,                    // delay no renders till flip (standard 0)
  ID_PRETEND_STENCIL,
  ID_ISP_HALF_OFFSET,
  ID_TA_HALF_OFFSET,
  ID_ENABLE_PVR_LOGO_D3D,                           // display PowerVR logo ingame
  ID_NOVSYNC_LOCK,                        // deactivate vert. sync. (standard 0)
  ID_DISABLE_24BIT_3DDEVICE,
  ID_DISABLE_ALPHA_TEST_D3D,
  ID_DISABLE_COPY_BLT3D,    
  ID_DISABLE_DXT1,
  ID_DISABLE_HW_CURSOR,    
  ID_DISABLE_MIPMAPPING,    
  ID_DISABLE_RENDER_ONLOCK,
  ID_DISABLE_STENCIL,
  ID_DONT_WAIT_ONLOCK,
  ID_ENABLE_COMPLEX_SCENE,
  ID_ENABLE_TEXTURE_MANAGEMENT,
  ID_ONLY_USE_LAST_FOG_COL,
  ID_RELOAD_REG_ON_SUBBLT,

 // OpenGL registry settings (order relevant)
  ID_COMPLEX_SCENE_MODE,                  // activate scene manager (standard 0)
  ID_ENABLE_FAST_COL,                     // activate fast color calculation (standard 0)
  ID_FORCE_SINGLE_BUFFERED,               // single buffer (standard 0)
  ID_HONOUR_DOUBLE_BUFFER,                // application buffering (standard 0)
  ID_IGNORE_SCISSOR,                      // deactivate scissor support (standard 0)
  ID_ENABLE_DXT1,                         // activate DXT1 compression (standard 1)
  ID_DISABLE_ANISO_TEX,
  ID_FORCE_ANISO_TEX,                     // activate anisotrop filtering (standard 0)
  ID_SCENE_AAX,                           // activate hor. antialias (standard 0)
  ID_SCENE_AAY,                           // activate vert. antialias (standard 0)
  ID_COMPRESS_1BIT_ALPHA_TEX,             // activate texture compression pt.1 (standard 0)  
  ID_COMPRESS_DXT35_TEX,
  ID_COMPRESS_LIGHTMAP,                   // (1)
  ID_COMPRESS_RGB_TEX,                    // activate texture compression pt.2 (standard 0)  
  ID_DEPTH_BUFFERLESS_HSR,                // activate depth buffer (standard 1)
  ID_DEPTH_STORE_DEFAULT,                 // store depth-buffer (standard 1)
  ID_SYNC_DEPTHBUFFER,                    // activate asynch. depth buffer (standard 1)
  ID_STENCIL_BUFFERLESS_SB,               // activate stencil buffer (standard 1)
  ID_STENCIL_STORE_DEFAULT,               // store stencil-buffer (standard 1)
  ID_SYNC_STENCILBUFFER,                   // activate asynch. stencil  buffer (standard 1)
  ID_ENABLE_PVR_LOGO_OGL, 
  ID_ENABLE_VSYNC,                        // activate vert. sync. (standard 1)
  ID_ALLOW_EXCLUSIVE_MODE,
  ID_CHECK_FOR_WINDOW_CHANGES,   
  ID_DISABLE_ALPHA_TEST_OGL,
  ID_HANDLE_ZTRICK,

  ID_DDRAW,
  ID_OTHER_D3D,
  ID_OTHER_OGL,

 // menu settings
  ID_NEW_ENTRY = wxID_HIGHEST+1,
  ID_DEL_ENTRY,
  ID_ADD_LAST,
  ID_GOTO_LAST,
  ID_CHILD_ENTRY,
  ID_FREE_CHILD,
  ID_DEL_CHILD,
  ID_REREAD_REG,
  ID_SAVE_REG,
  ID_LOAD_REG,
  ID_DEMO_MODE,
  ID_ABOUT,
  ID_HELP,
  ID_QUIT,

 // buttons
  ID_START,
  ID_PATH_BOX,
  ID_GAME_LABEL,
  ID_APPLY,
  ID_RESET,

 // frame items
  ID_DROPDOWN,

 // tab settings
  ID_NOTEBOOK
};

// **************************
// Class Declaration
// **************************

class BasicFrame : public wxFrame {       // main window class    
  public:
	BasicFrame( const wxChar *title, long style=wxDEFAULT_FRAME_STYLE,
                int width=-1, int height=-1, int xpos=-1, int ypos=-1 );        // constructor

  private:
    wxChoice* dropDown;                      // drop Down menu with game entries
    wxTextCtrl* gameLabel;                    // Label for game
    wxCheckBox* pathBox;                    // Checkbox if path is set
    wxTextCtrl* childFileLabel;              // Label for "is parent of"
    wxTextCtrl* parentFileLabel;             // Label for "is child of"
    wxButton* startButton;                   // Start-Application Button
    wxButton* applyButton;                   // Apply Button
    wxButton* resetButton;                   // Reset Button

    wxMenu* entryMenu;                       // entry menu (new, delete, ...)
    wxMenu* childMenu;                       // child menu (add, delete, free, ...)
    wxMenu* optionMenu;                      // options menu
    wxMenu* helpMenu;                        // help menu
        
    wxNotebook* layout;                   // layout for D3D and OGL tabs
    wxPanel* tabD3D;                      // D3D tab
    wxPanel* tabOGL;                      // OGL tab
    wxString currentTab;                  // stores the current tab
    
    wxCheckBox* RegBox[64];               // checkboxes for reg settings
    wxCheckBox* otherRegBoxDDraw;         // checkbox for showing a DirectDraw entry
    wxCheckBox* otherRegBoxD3D;           // checkbox for other reg settings in Direct3D
    wxCheckBox* otherRegBoxOGL;           // checkbox for other reg settings in OpenGL
    wxArrayString list;                   // array of strings with all registry profiles (filenames)
    wxArrayString listD3D, listOGL;       // registry profiles for D3D and OGL

    wxString regData;                     // string for reading registry data
    wxRegKey* regKey;                     // key for handling the registry
    wxString driverPos;                   // driver position in registry

    wxString lastGameType;                // last game type started
    wxString lastApp;                     // last application started
    
    bool entryChanged;                    // variable for checking if something has changed
    bool demoMode;                        // demo mode for non-Kyro cards (cannot be toggled)
                                          // demo mode for Kyro cards through menu item "Demo Mode"

    void redrawChoice(wxString string);        // redraws all settings on the screen
    void redrawChoice(int n=0);                // see above
    void rdC(void);                            // redraws all settings on the screen (internal)
    void redrawChoiceList(wxString string);    // re-reads the current list in the choice field
    void redrawChoiceList(int n=0);            // see above
    void rdCL(void);                           // re-reads the current list in the choice field (internal)

    void newEntry (wxCommandEvent & event);             // new entry want to be made
    void delEntry (wxCommandEvent & event);             // entry should be deleted    
    void gotoLastEntry (wxCommandEvent & event);        // goto last used application
    void addLastEntry (wxCommandEvent & event);         // add last used application
    void addChild (wxCommandEvent & event);              // add child
    void delChild (wxCommandEvent & event);              // delete child (pre-function)
    void dChild (wxString child, wxString parent);       // delete children recursively (internal)
    void freeChild (wxCommandEvent & event);              // free child
    void resetEntry (wxCommandEvent & event);             // call redrawChoice
    void applyChanges (wxCommandEvent & event);           // write everything to registry and call redrawChoice  
    void aChanges (wxString game);                        // add reg keys recursively (internal)
    void selectPath (wxCommandEvent & event);              // select a path where the game is located
    void rereadReg (wxCommandEvent & event);              // re-read the registry settings
    void loadReg (wxCommandEvent & event);                // load registry settings from file
    void saveReg (wxCommandEvent & event);                // save registry settings to file
    void toggleDemoMode (wxCommandEvent & event);         // re-read the registry settings
    void quitApp (wxCommandEvent & event);                // quit application
    void help (wxCommandEvent & event);                  // show help dialog
    void about (wxCommandEvent & event);                  // show about dialog

    void startApp( wxCommandEvent & event);               // start application
    void gameChanged( wxCommandEvent & event);            // game name has been changed
    void checkBoxChange( wxCommandEvent & event);         // some checkbox has been clicked
    void ChoiceChanged (wxCommandEvent & event);          // some choice has been selected

    DECLARE_EVENT_TABLE()
};

class BasicApplication : public wxApp {                   // application class
  public:
    virtual bool OnInit();                                // init (create frame etc.)

  private:  
    void SetConsts(void);                                 // set strings for RegSet
};

static void message(wxString string);        // messagebox for internal use only

static int CompareStringNocase(const wxString& first, const wxString& second);       // compares to strings for sorting

// *****************************
// Event-Handler & Application
// *****************************


BEGIN_EVENT_TABLE (BasicFrame, wxFrame)
 // menu events
  EVT_MENU(ID_NEW_ENTRY, BasicFrame::newEntry)                   // new game entry
  EVT_MENU(ID_DEL_ENTRY, BasicFrame::delEntry)                   // del game entry
  EVT_MENU(ID_GOTO_LAST, BasicFrame::gotoLastEntry)              // goto last used application
  EVT_MENU(ID_ADD_LAST, BasicFrame::addLastEntry)                 // add last used application
  EVT_MENU(ID_CHILD_ENTRY, BasicFrame::addChild)                  // add child entry
  EVT_MENU(ID_FREE_CHILD, BasicFrame::freeChild)                   // free child
  EVT_MENU(ID_DEL_CHILD, BasicFrame::delChild)                    // del child
  EVT_MENU(ID_QUIT, BasicFrame::quitApp)                           // quit application
  EVT_MENU(ID_REREAD_REG, BasicFrame::rereadReg)                   // reread registry to list
  EVT_MENU(ID_SAVE_REG, BasicFrame::saveReg)                       // save registry to file
  EVT_MENU(ID_LOAD_REG, BasicFrame::loadReg)                       // load registry from file
  EVT_MENU(ID_DEMO_MODE, BasicFrame::toggleDemoMode)               // toggles demo mode
  EVT_MENU(ID_HELP, BasicFrame::help)                            // show help dialog
  EVT_MENU(ID_ABOUT, BasicFrame::about)                            // show about dialog
  
  EVT_BUTTON(ID_APPLY, BasicFrame::applyChanges)               // apply changes and write to registry
  EVT_BUTTON(ID_RESET, BasicFrame::resetEntry)                 // reset changes and read from registry
  EVT_BUTTON(ID_START, BasicFrame::startApp)                    // start application

  EVT_TEXT(ID_GAME_LABEL, BasicFrame::gameChanged)                  // game name has been changed
  EVT_NOTEBOOK_PAGE_CHANGED(ID_NOTEBOOK, BasicFrame::rereadReg)      // the tab has changed
  EVT_CHOICE(ID_DROPDOWN, BasicFrame::ChoiceChanged)                 // some file in the drop down list has been selectedt

  EVT_CHECKBOX(ID_DDRAW, BasicFrame::checkBoxChange)            
  EVT_CHECKBOX(ID_PATH_BOX, BasicFrame::selectPath)

 // D3D events  
  EVT_CHECKBOX(ID_FORCE_LARGE_TEX_TO_DXT1, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_USE_RHW, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ZLOAD, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ZSTORE, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ZTRANSLATE, BasicFrame::checkBoxChange)  
  EVT_CHECKBOX(ID_ENABLE_SCENESS, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_XSS, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_YSS, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_SOFTPT, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_TEXTURESS, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_ANISO, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_TEXTURE_TLF, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_FAST_TLF, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DONT_DISABLE_FAST_TLF, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_ENTNL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_MID_SCENE, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DXT_EXPOSE_CAPS, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_EXPOSE_CUBEMAP_CAPS, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_EXPOSE_2KTEXTURES, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_NORENDERS_TILL_FLIP, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_PRETEND_STENCIL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ISP_HALF_OFFSET, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_TA_HALF_OFFSET, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_PVR_LOGO_D3D, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_NOVSYNC_LOCK, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_24BIT_3DDEVICE, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_ALPHA_TEST_D3D, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_COPY_BLT3D, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_DXT1, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_HW_CURSOR, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_MIPMAPPING, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_RENDER_ONLOCK, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_STENCIL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DONT_WAIT_ONLOCK, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_COMPLEX_SCENE, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_TEXTURE_MANAGEMENT, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ONLY_USE_LAST_FOG_COL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_RELOAD_REG_ON_SUBBLT, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_OTHER_D3D, BasicFrame::checkBoxChange)

 // OGL events
  EVT_CHECKBOX(ID_COMPLEX_SCENE_MODE, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_FAST_COL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_FORCE_SINGLE_BUFFERED, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_HONOUR_DOUBLE_BUFFER, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_IGNORE_SCISSOR, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_DXT1, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_ANISO_TEX, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_FORCE_ANISO_TEX, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_SCENE_AAX, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_SCENE_AAY, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_COMPRESS_1BIT_ALPHA_TEX, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_COMPRESS_DXT35_TEX, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_COMPRESS_LIGHTMAP, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_COMPRESS_RGB_TEX, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DEPTH_BUFFERLESS_HSR, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DEPTH_STORE_DEFAULT, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_SYNC_DEPTHBUFFER, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_STENCIL_BUFFERLESS_SB, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_STENCIL_STORE_DEFAULT, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_SYNC_STENCILBUFFER, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_PVR_LOGO_OGL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ENABLE_VSYNC, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_ALLOW_EXCLUSIVE_MODE, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_CHECK_FOR_WINDOW_CHANGES, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_DISABLE_ALPHA_TEST_OGL, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_HANDLE_ZTRICK, BasicFrame::checkBoxChange)
  EVT_CHECKBOX(ID_OTHER_OGL, BasicFrame::checkBoxChange)

END_EVENT_TABLE()

IMPLEMENT_APP(BasicApplication)

// **************************
// Function definitions
// **************************

static void message(wxString string) {        // messagebox for internal use only
  wxMessageBox( string, "Report", wxOK );
}

static int CompareStringNocase(const wxString& first, const wxString& second) {
    return (first.CmpNoCase(second));
}

BasicFrame::BasicFrame( const wxChar *title, long style, int width, int height, int xpos, int ypos)
: wxFrame( (wxFrame *) NULL, wxID_ANY, title, wxPoint(xpos, ypos), wxSize(width, height), style) {

  wxImage::AddHandler(new wxJPEGHandler);                   // add JPEG-support

  // Check if there is a Kyro card is installed
  regKey=new wxRegKey("HKEY_LOCAL_MACHINE\\SOFTWARE\\PowerVR\\KYRO");
  if (!regKey->Exists() || !regKey->HasValue("LatestDrvReg") || !regKey->QueryValue("LatestDrvReg", regData)) {
    wxMessageBox( "There seems no PowerVR Kyro graphics adapter to be installed.\nApplication will run in demo mode.", "Error: No PowerVR Kyro detected.", wxOK | wxICON_ERROR  );
    demoMode=true;
    SetTitle(name+" version "+version+" (Demo Mode)");
  }
  if (!demoMode) {
    driverPos="HKLM\\"+regData+"\\KYRO\\Game Settings";          // setting driver position
    regKey=new wxRegKey(driverPos);
    if (!regKey->Exists()) {                                     // driver position is wrong
      wxMessageBox( "There seems no PowerVR Kyro graphics adapter to be installed.\nApplication will run in demo mode.", "Error: No PowerVR Kyro detected.", wxOK | wxICON_ERROR  );
      demoMode=true;
      SetTitle(name+" version "+version+" (Demo Mode)");
    }                           
    else {
      driverPos=regKey->GetName(false);                           // gets the full registry key name
    }
  }
  
  lastGameType="";                                       // last game type started
  lastApp="";                                          // last application started
  
  entryMenu=new wxMenu();
  entryMenu->Append(ID_NEW_ENTRY, "&New Entry...\tCtrl+N", "Add new file to the registry.");
  entryMenu->Append(ID_DEL_ENTRY, "&Delete Entry\tDelete", "Delete current entry from registry.");
  entryMenu->AppendSeparator();
  entryMenu->Append(ID_ADD_LAST, "Add &Last Application\tCtrl+H", "Add last started game to the registry.");
  entryMenu->Append(ID_GOTO_LAST, "&Goto Last Application\tCtrl+G", "Jump to the entry for the last started game.");
  entryMenu->AppendSeparator();
  entryMenu->Append(ID_QUIT, "&Quit\tAlt+F4", "Quit control panel.");

  childMenu=new wxMenu();
  childMenu->Append(ID_CHILD_ENTRY, "&Add Child...\tCtrl+A", "Add new child to current entry.");
  childMenu->Append(ID_FREE_CHILD, "&Free Child\tCtrl+F", "Delete parent/child association.");
  childMenu->Append(ID_DEL_CHILD, "&Delete Child\tCtrl+D", "Delete child entry from registry.");

  optionMenu=new wxMenu();
  optionMenu->Append(ID_REREAD_REG, "&Reread Registry\tF3", "Read all registry entries again.");
  optionMenu->AppendSeparator();
  optionMenu->Append(ID_SAVE_REG, "&Save Registry...\tF5", "Make a backup of your registry.");
  optionMenu->Append(ID_LOAD_REG, "&Load Registry...\tF7", "Load a registry backup from HD.");
  optionMenu->AppendSeparator();
  optionMenu->AppendCheckItem(ID_DEMO_MODE, "&Demo Mode\tF9", "Toogle Demo Mode.");
  if (demoMode) optionMenu->Check(ID_DEMO_MODE, true);

  helpMenu=new wxMenu();
  helpMenu->Append(ID_HELP, "&Help\tF1", "Show help dialog.");
  helpMenu->AppendSeparator();
  helpMenu->Append(ID_ABOUT, "&About", "Show about dialog.");

  wxMenuBar* menuBar=new wxMenuBar();
  menuBar->Append(entryMenu, "&Games");
  menuBar->Append(childMenu, "&Children");
  menuBar->Append(optionMenu, "&Options");
  menuBar->Append(helpMenu, "&Help");
  SetMenuBar(menuBar);        

  SetBackgroundColour(wxColour(212,208,199));         // sets background color to a light gray
  SetForegroundColour(wxColour(0,0,0));               // sets foreground color to black      
  CreateStatusBar(1);                                 // creates the status bar with one field   

 // drop down choice (empty at first)
  wxString tempList[]={};
  dropDown=new wxChoice(this, ID_DROPDOWN, wxDefaultPosition, wxSize(160,-1), 0, tempList);

 // buttons for top menu 
  gameLabel = new wxTextCtrl(this, ID_GAME_LABEL, "", wxDefaultPosition, wxSize(300,-1));
  pathBox = new wxCheckBox(this, ID_PATH_BOX, "");
  startButton=new wxButton(this, ID_START, "     Start\nApplication", wxDefaultPosition, wxSize(148,35));
  childFileLabel = new wxTextCtrl(this, wxID_ANY, "(None)", wxDefaultPosition, wxSize(160,-1), wxTE_READONLY);
  parentFileLabel = new wxTextCtrl(this, wxID_ANY, "(None)", wxDefaultPosition, wxSize(160,-1), wxTE_READONLY);

 // buttons for bottom menu
  applyButton=new wxButton(this, ID_APPLY, "Apply", wxDefaultPosition, wxSize(150,-1));
  applyButton->Disable();
  resetButton=new wxButton(this, ID_RESET, "Reset", wxDefaultPosition, wxSize(150,-1));
  resetButton->Disable();

 // grid sizer for top menu
  wxStaticBoxSizer* gridSizerTopMenu = new wxStaticBoxSizer(new wxStaticBox(this, wxID_ANY, "Game Info"), wxVERTICAL);

  wxBoxSizer *gridSizerTopHor1, *gridSizerTopHor2, *gridSizerTopVer1, *gridSizerTopVer2, *gridSizerTopVer3;

  gridSizerTopVer1 = new wxBoxSizer(wxVERTICAL);
  gridSizerTopVer1->Add(new wxStaticText(this, wxID_ANY, "Game", wxDefaultPosition), 0, wxBOTTOM, 3);
  gridSizerTopVer1->Add(gameLabel, 0, wxBOTTOM, 3);

  gridSizerTopVer2 = new wxBoxSizer(wxVERTICAL);
  gridSizerTopVer2->Add(new wxStaticText(this, wxID_ANY, "Path", wxDefaultPosition), 0, wxBOTTOM, 3);
  gridSizerTopVer2->Add(pathBox, 0, wxBOTTOM, 3);

  gridSizerTopHor1 = new wxBoxSizer(wxHORIZONTAL);
  gridSizerTopHor1->Add(gridSizerTopVer1, 0, wxLEFT | wxRIGHT , 5);
  gridSizerTopHor1->Add(gridSizerTopVer2, 0, wxLEFT | wxRIGHT , 5);
  gridSizerTopHor1->Add(startButton, 0, wxLEFT | wxRIGHT | wxALIGN_CENTER, 5);

  gridSizerTopVer1 = new wxBoxSizer(wxVERTICAL);
  gridSizerTopVer1->Add(new wxStaticText(this, wxID_ANY, "Filename", wxDefaultPosition), 0, wxBOTTOM, 3);
  gridSizerTopVer1->Add(dropDown, 0, wxBOTTOM, 3);

  gridSizerTopVer2 = new wxBoxSizer(wxVERTICAL);
  gridSizerTopVer2->Add(new wxStaticText(this, wxID_ANY, "Is Parent of", wxDefaultPosition), 0, wxBOTTOM, 3);
  gridSizerTopVer2->Add(childFileLabel, 0, wxBOTTOM, 3);

  gridSizerTopVer3 = new wxBoxSizer(wxVERTICAL);
  gridSizerTopVer3->Add(new wxStaticText(this, wxID_ANY, "Is Child of", wxDefaultPosition), 0, wxBOTTOM, 3);
  gridSizerTopVer3->Add(parentFileLabel, 0, wxBOTTOM, 3);

  gridSizerTopHor2 = new wxBoxSizer(wxHORIZONTAL);
  gridSizerTopHor2->Add(gridSizerTopVer1, 0, wxLEFT | wxRIGHT , 5);
  gridSizerTopHor2->Add(gridSizerTopVer2, 0, wxRIGHT , 5);
  gridSizerTopHor2->Add(gridSizerTopVer3, 0, wxRIGHT, 5);

  gridSizerTopMenu->Add(gridSizerTopHor1, 0, wxBOTTOM, 3);
  gridSizerTopMenu->Add(gridSizerTopHor2, 0, wxBOTTOM, 3);

 // grid sizer for bottom menu
  wxFlexGridSizer* gridSizerBottomMenu=new wxFlexGridSizer(1,2,5,10);
  gridSizerBottomMenu->Add(resetButton);
  gridSizerBottomMenu->Add(applyButton);

 // layout with both tabs for D3D and OGL
  layout = new wxNotebook(this, ID_NOTEBOOK, wxPoint(-1,-1),wxSize(576,380));

 // GridSizers for the tabs
  wxStaticBoxSizer *gridSizerUsual, *gridSizerQuality, *gridSizerKyro, *gridSizerMisc, *gridSizerDSBuffer, *gridSizerCompression;
  wxBoxSizer *gridSizerOthers0, *gridSizerOthers1, *gridSizerOthers2, *gridSizerSettings;

  tabD3D = new wxPanel(layout);                       // Direct3D tab panel
  for (int i=0; i<38; i++)
    RegBox[i]=new wxCheckBox(tabD3D, i, RegSet[i]);   // add D3D checkboxes 
  otherRegBoxD3D=new wxCheckBox(tabD3D, ID_OTHER_D3D, "Non-specific reg settings");
  otherRegBoxDDraw = new wxCheckBox(tabD3D, ID_DDRAW, "Direct Draw");

 // grid sizer for kyro driver settings
  gridSizerKyro = new wxStaticBoxSizer(new wxStaticBox(tabD3D, wxID_ANY, "Kyro Driver Settings"), wxVERTICAL);
  for (int i=0; i<5 ; i++)
    gridSizerKyro->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerKyro->Add(160,0);

 // grid sizer for quality settings
  gridSizerQuality = new wxStaticBoxSizer(new wxStaticBox(tabD3D, wxID_ANY, "Quality Settings"), wxVERTICAL);
  for (int i=5; i<14 ; i++)
    gridSizerQuality->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerQuality->Add(160,0);

 // grid sizer for usual settings
  gridSizerUsual = new wxStaticBoxSizer(new wxStaticBox(tabD3D, wxID_ANY, "Usual Settings"), wxVERTICAL);
  for (int i=14; i<23 ; i++)
    gridSizerUsual->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerUsual->Add(160,0); 

 // grid sizer for misc settings
  gridSizerMisc = new wxStaticBoxSizer(new wxStaticBox(tabD3D, wxID_ANY, "Misc Settings"), wxVERTICAL);
  for (int i=23; i<25 ; i++)
    gridSizerMisc->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerMisc->Add(160,10);
  gridSizerMisc->Add(otherRegBoxDDraw, 0, wxBOTTOM, 5);

 // grid sizer for other settings 0 (kyro + quality)
  gridSizerOthers0 = new wxBoxSizer(wxVERTICAL);
  gridSizerOthers0->Add(gridSizerKyro, 0, wxBOTTOM, 10);
  gridSizerOthers0->Add(gridSizerQuality, 0, wxBOTTOM, 10);

 // grid sizer for other settings 1 (usual + misc)
  gridSizerOthers1 = new wxBoxSizer(wxVERTICAL);
  gridSizerOthers1->Add(gridSizerUsual, 0, wxBOTTOM, 10);
  gridSizerOthers1->Add(gridSizerMisc, 0, wxBOTTOM, 10);

 // grid sizer for other settings 2
  gridSizerOthers2 = new wxStaticBoxSizer(new wxStaticBox(tabD3D, wxID_ANY, "Other Settings"), wxVERTICAL);
  for (int i=25; i<38 ; i++)
    gridSizerOthers2->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerOthers2->Add(160,10);
  gridSizerOthers2->Add(otherRegBoxD3D);

 // grid sizer for the whole D3D tab
  gridSizerSettings=new wxBoxSizer(wxHORIZONTAL);
  gridSizerSettings->Add(gridSizerOthers0, 0, wxALL, 10);
  gridSizerSettings->Add(gridSizerOthers1, 0, wxALL, 10);
  gridSizerSettings->Add(gridSizerOthers2, 0, wxALL, 10);
  
  tabD3D->SetSizer(gridSizerSettings);                  // set the D3D sizer

  tabOGL = new wxPanel(layout);                        // OpenGL tab panel
  for (int i=38; i<64; i++)
    RegBox[i]=new wxCheckBox(tabOGL, i, RegSet[i]);    // add OGL check boxes
  otherRegBoxOGL=new wxCheckBox(tabOGL, ID_OTHER_OGL, "Non-specific reg settings");

 // grid sizer for kyro driver settings
  gridSizerKyro = new wxStaticBoxSizer(new wxStaticBox(tabOGL, wxID_ANY, "Kyro Driver Settings"), wxVERTICAL);
  for (int i=38; i<43 ; i++)
    gridSizerKyro->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerKyro->Add(160,0);

 // grid sizer for quality settings
  gridSizerQuality = new wxStaticBoxSizer(new wxStaticBox(tabOGL, wxID_ANY, "Quality Settings"), wxVERTICAL);
  for (int i=43; i<48 ; i++)
    gridSizerQuality->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerQuality->Add(160,0);

 // grid sizer for compression settings
  gridSizerCompression = new wxStaticBoxSizer(new wxStaticBox(tabOGL, wxID_ANY, "Compression Settings"), wxVERTICAL);
  for (int i=48; i<52 ; i++)
    gridSizerCompression->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerCompression->Add(160,0);

 // grid sizer for depth/stencil buffer settings
  gridSizerDSBuffer = new wxStaticBoxSizer(new wxStaticBox(tabOGL, wxID_ANY, "Depth/Stencil Buffer Settings"), wxVERTICAL);
  for (int i=52; i<58 ; i++)
    gridSizerDSBuffer->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerDSBuffer->Add(160,0);

 // grid sizer for misc settings
  gridSizerMisc = new wxStaticBoxSizer(new wxStaticBox(tabOGL, wxID_ANY, "Misc Settings"), wxVERTICAL);
  for (int i=58; i<60 ; i++)
    gridSizerMisc->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerMisc->Add(160,0);

 // grid sizer for other settings 0 (kyro + quality + compression)
  gridSizerOthers0 = new wxBoxSizer(wxVERTICAL);
  gridSizerOthers0->Add(gridSizerKyro, 0, wxBOTTOM, 10);
  gridSizerOthers0->Add(gridSizerQuality, 0, wxBOTTOM, 10);
  gridSizerOthers0->Add(gridSizerCompression, 0, wxBOTTOM, 10);

 // grid sizer for other settings 1 (DSbuffer + misc)
  gridSizerOthers1 = new wxBoxSizer(wxVERTICAL);
  gridSizerOthers1->Add(gridSizerDSBuffer, 0, wxBOTTOM, 10);
  gridSizerOthers1->Add(gridSizerMisc, 0, wxBOTTOM, 10);

 // grid sizer for other settings 2
  gridSizerOthers2 = new wxStaticBoxSizer(new wxStaticBox(tabOGL, wxID_ANY, "Other Settings"), wxVERTICAL);
  for (int i=60; i<64 ; i++)
    gridSizerOthers2->Add(RegBox[i], 0, wxBOTTOM, 5);
  gridSizerOthers2->Add(160,10);
  gridSizerOthers2->Add(otherRegBoxOGL);

 // grid sizer for the whole OGL tab
  gridSizerSettings=new wxBoxSizer(wxHORIZONTAL);
  gridSizerSettings->Add(gridSizerOthers0, 0, wxALL, 10);
  gridSizerSettings->Add(gridSizerOthers1, 0, wxALL, 10);
  gridSizerSettings->Add(gridSizerOthers2, 0, wxALL, 10);

  tabOGL->SetSizer(gridSizerSettings);             // set the OGL sizer 

  layout->AddPage(tabD3D,"Direct3D");              // add D3D panel to the notebook
  layout->AddPage(tabOGL,"OpenGL");                // add OGL panel to the notebook
  
 // grid sizer for the whole frame
  wxBoxSizer *gridSizerAll = new wxBoxSizer(wxVERTICAL);
  gridSizerAll->Add(gridSizerTopMenu, 0, wxBOTTOM | wxLEFT | wxRIGHT, 10);
  gridSizerAll->Add(layout, 0, wxBOTTOM | wxLEFT | wxRIGHT, 10);
  gridSizerAll->Add(gridSizerBottomMenu, 0, wxBOTTOM | wxLEFT | wxRIGHT, 10);
  SetSizer(gridSizerAll);              // sets the sizer

};

void BasicFrame::rdC(void) {             // redraws all settings on the screen (internal)
  entryChanged=false;                          // no changes until now
  applyButton->Disable();                      // ... so there is no need for applying anything
  resetButton->Disable();                      // ... or hit reset   

  // **************************************************
  // Setting standard settings for menu and checkboxes
  // **************************************************

  // the following may be changed after reading the registry settings for this entry

  parentFileLabel->SetValue("(None)");          // no parent file
  childFileLabel->SetValue("(None)");           // no child file
  gameLabel->SetValue("");                      // no game name 
  pathBox->SetValue(false);                     // uncheck path box
  startButton->Disable();                       // disable start button
  
  entryMenu->SetLabel(ID_ADD_LAST, "Add &Last Application\tCtrl+H");
  entryMenu->Enable(ID_ADD_LAST, false);
  entryMenu->SetLabel(ID_GOTO_LAST, "&Goto Last Application\tCtrl+G");
  entryMenu->Enable(ID_GOTO_LAST, false);
  
  if (list.GetCount()==0) {                            // if the list is empty
    tabD3D->Disable();             // disable panels
    tabOGL->Disable();
  } 
  else {
    tabD3D->Enable();             // enable panels
    tabOGL->Enable();
  }
  childMenu->Enable(ID_CHILD_ENTRY, true);            // enable items for adding children
  childMenu->Enable(ID_DEL_CHILD, false);             // disable items for deleting children...
  childMenu->Enable(ID_FREE_CHILD, false);            // ... free children

  // standard registry settings
  if (currentTab=="D3D") {                            // Direct3D
    for (int i=0; i<38; i++) {
      switch (i) {
        case ID_DISABLE_XSS:                          // deactivate hor. antialias (standard 1)
        case ID_DISABLE_YSS:                          // deavtivate vert. antialias (standard 1)
            RegBox[i]->Disable(); 
            RegBox[i]->SetValue(true); break;     
        case ID_DISABLE_ENTNL:                        // deactivate Enhanced TnL (standard 1) 
            RegBox[i]->SetValue(true); break;     
        default: RegBox[i]->SetValue(false); break;    
      }           
    }
    regKey=new wxRegKey(driverPos+"\\DDraw\\"+dropDown->GetStringSelection());         // game reg folder
    if (regKey->Exists()) otherRegBoxDDraw->SetValue(true);
    else otherRegBoxDDraw->SetValue(false);
    otherRegBoxD3D->SetValue(false);
    regKey=new wxRegKey(driverPos+"\\D3D\\"+dropDown->GetStringSelection());         // game reg folder
  }  
  else if (currentTab=="OpenGL") {                   // OpenGL
    for (int i=38; i<64; i++) {
      switch (i) {
        case ID_ENABLE_DXT1:                         // activate DXT1 compression (standard 1)
        case ID_DEPTH_BUFFERLESS_HSR:                // activate depth buffer (standard 1)
        case ID_DEPTH_STORE_DEFAULT:                 // store depth-buffer (standard 1)
        case ID_SYNC_DEPTHBUFFER:                    // activate asynch. depth buffer (standard 1)
        case ID_STENCIL_BUFFERLESS_SB:               // activate stencil buffer (standard 1)
        case ID_STENCIL_STORE_DEFAULT:               // store stencil-buffer (standard 1)
        case ID_SYNC_STENCILBUFFER:                  // activate asynch. stencil  buffer (standard 1)
        case ID_ENABLE_VSYNC:                        // activate vert. sync. (standard 1)
        case ID_COMPRESS_LIGHTMAP:                   // (1)
            RegBox[i]->SetValue(true); break;    
        default: RegBox[i]->SetValue(false); break;    
      }           
    }           
    otherRegBoxOGL->SetValue(false);
    regKey=new wxRegKey(driverPos+"\\OpenGL\\"+dropDown->GetStringSelection());         // game reg folder 
  }
  
  // **********************************************
  // Reading registry and setting checkboxes etc.
  // **********************************************

  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {    // only access registry in non demo mode
    wxString tempData; int tempInt; long tempLong;         // tempory variables                               
    if (regKey->GetFirstValue(regData, tempLong)) {              // read first reg entry ...
      do {
        tempInt=RegSet.Index(regData, false);                // search entry in RegSet-Array
        if (tempInt!=wxNOT_FOUND) {
          regKey->QueryValue(regData, tempData);             // query value 
      
          // message("|"+regKey->GetName()+"|"+regData+"|"+tempData+"|");        // internal use only
      
          if (tempData=="0") RegBox[tempInt]->SetValue(false);          // check boxes ...
          else if (tempData=="1") RegBox[tempInt]->SetValue(true);      // ...or uncheck them
          if (tempInt==ID_ENABLE_SCENESS) {               // antialias is activated
            RegBox[ID_DISABLE_XSS]->Enable();       // enable hor. antialias checkbox
            RegBox[ID_DISABLE_YSS]->Enable();       // enable vert. antialias checkbox
          }
        }
        else {                                  // entry is no (supported) reg setting or UIDummy    
          if (regData=="UIDummyName") {                        // game entry name
            regKey->QueryValue(regData, tempData);             // query value 
            gameLabel->SetValue(tempData);                     // set game name 
          }
          else if (regData=="UIDummyPath") {                   // game entry path
            pathBox->SetValue(true);                           // check path box
            startButton->Enable();                             // enable start button
          }
          else if (regData=="UIDummyChild") {                   // game entry is parent
            regKey->QueryValue(regData, tempData);             // query value 
            childFileLabel->SetValue(tempData);                  // set child label to the filename
            childMenu->Enable(ID_CHILD_ENTRY, false);            // disable items for adding children
            childMenu->Enable(ID_DEL_CHILD, true);               // enable items for deleting children...
            childMenu->Enable(ID_FREE_CHILD, true);              // ... free children
          }
          else if (regData=="UIDummyParent") {             // game entry is child (no changes can be made)
            regKey->QueryValue(regData, tempData);             // query value 
            parentFileLabel->SetValue(tempData);                  // set parent label
            tabD3D->Disable();                               // disable panels so no changes can be made                     
            tabOGL->Disable();
          }
          else if (regData!="DuplicateAppName" && !regData.StartsWith("UI")) {
               otherRegBoxD3D->SetValue(true);
               otherRegBoxOGL->SetValue(true);
          }     
        }
      } while (regKey->GetNextValue(regData, tempLong));       // ...until there is no more
    }

    regKey=new wxRegKey("HKEY_LOCAL_MACHINE\\SOFTWARE\\PowerVR\\KYRO");
    regKey->QueryValue("LatestDrvReg", regData);
    regKey=new wxRegKey("HKLM\\"+regData+"\\KYRO\\LastApp");
    regKey->QueryValue("LastGameType", regData);
    if (regData.Length()==1) {                                  // D3D Game
      lastGameType="D3D";                                       // last game type started
      regKey->QueryValue("LastD3DApp", regData);
      lastApp="";                                          // last application started
      const char* tstr=regData.c_str();
      for (int i=0; i<30 && tstr[i]!='\0'; i+=2)
        lastApp+=tstr[i];
    }
    else {                                                      // OGL Game
      lastGameType="OpenGL";                                    // last game type started
      regKey->QueryValue("LastOpenGLApp", regData);
      lastApp=regData;                                          // last application started
    }
    if (!lastApp.IsEmpty()) {                                  // show last app
      entryMenu->SetLabel(ID_ADD_LAST, "Add &Last Application: "+lastApp+" ("+lastGameType+")\tCtrl+H");    // set labels
      entryMenu->SetLabel(ID_GOTO_LAST, "&Goto Last Application: "+lastApp+" ("+lastGameType+")\tCtrl+G");

      if (currentTab=="D3D") {
        if (list.Index(lastApp, false)==wxNOT_FOUND && listOGL.Index(lastApp, false)==wxNOT_FOUND) entryMenu->Enable(ID_ADD_LAST, true);   // only show add item (item not in list)
        else entryMenu->Enable(ID_GOTO_LAST, true);                                          // only show goto item (item in list)
      }
      else  if (currentTab=="OpenGL") {
        if (list.Index(lastApp, false)==wxNOT_FOUND && listD3D.Index(lastApp, false)==wxNOT_FOUND) entryMenu->Enable(ID_ADD_LAST, true);   // only show add item (item not in list)
        else entryMenu->Enable(ID_GOTO_LAST, true);                                          // only show goto item (item in list)
      }
    }
  }
};

void BasicFrame::redrawChoice(wxString string) {             // redraws all settings on the screen

  dropDown->SetStringSelection(string);        // sets the drop down field to the right selection
  rdC();             // redraws all settings on the screen (internal)
};

void BasicFrame::redrawChoice(int n) {   // see above
  dropDown->SetSelection(n);
  rdC();
};

void BasicFrame::rdCL(void) {   // re-reads the current list in the choice field (internal)

  dropDown->Clear();                                   // clear all entries in the drop down field
  if (list.GetCount()==0) {                            // if the list is empty
    dropDown->Disable();                               // ... disable the drop down field and ...
    entryMenu->Enable(ID_DEL_ENTRY, false);              // disable items for deleting entries...
    childMenu->Enable(ID_CHILD_ENTRY, false);            // ... adding children
  }  
  else {                                               // the list is not empty
    for (unsigned i=0; i<list.GetCount(); i++)         // add all list entries in the drop down field
      dropDown->Append(list.Item(i));
    dropDown->Enable();                               // ... enable the drop down field and ...
    entryMenu->Enable(ID_DEL_ENTRY, true);              // enable items for deleting entries...
    childMenu->Enable(ID_CHILD_ENTRY, true);            // ... adding children
  }
}

void BasicFrame::redrawChoiceList(wxString string) {   // re-reads the current list in the choice field

  rdCL();                                  // re-reads the current list in the choice field (internal)
  redrawChoice(string);                                // redraw all settings on the screen
};

void BasicFrame::redrawChoiceList(int n) {    // see above
  rdCL(); 
  redrawChoice(n);
};

void BasicFrame::ChoiceChanged (wxCommandEvent & event) { // some filename has been selected

  // TODO: Question for applying changes

  redrawChoice(dropDown->GetSelection());                 // redraw all settings on the screen 
};

void BasicFrame::newEntry (wxCommandEvent & event) {        // new game profile entry
  
  wxString tempString;                                       // temporary variables
  wxFileDialog* openFileDialog = new wxFileDialog (this, "Choose a file", "", "", "*.*", wxOPEN | wxCHANGE_DIR );    // file dialog for opening files
  if (openFileDialog->ShowModal() == wxID_OK) {              // file has been chosen
    if (currentTab=="OpenGL")
      tempString=openFileDialog->GetFilename();                 // OpenGL reg settings recognizes full file name
    else tempString=(openFileDialog->GetFilename()).Left(15);   // D3D reg settings recognizes onyl first 15 letters of file name
   
    if (list.Index(tempString, FALSE)==wxNOT_FOUND) {    // there is no entry with the same filename                                

      if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
        regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+tempString);         // new game reg folder 
        if (regKey->Create()) {                                       // create reg key
          regKey->SetValue("UIDummyPath", openFileDialog->GetDirectory());      // create path entry
          if (currentTab=="D3D") {
            regKey=new wxRegKey(driverPos+"\\DDraw\\"+tempString);        // game reg folder for DDraw
            if (!regKey->Exists()) regKey->Create();
          }  
          list.Add(tempString);                                    // add file to the list
          list.Sort(CompareStringNocase);                      // sort list alphabetically   
          redrawChoiceList(tempString);                      // put the list entries in the drop down field   
        } 
      }
      else {
        list.Add(tempString);                                    // add file to the list
        list.Sort(CompareStringNocase);                      // sort list alphabetically   
        redrawChoiceList(tempString);                      // put the list entries in the drop down field   
      }           
    } 
    else {
      wxMessageBox( "There is already a profil with this filename.\nPlease choose another file.", "Error: Duplicate Filename", wxOK | wxICON_ERROR  );
    }
  }
};

void BasicFrame::dChild (wxString child, wxString parent) {     // delete children recursively

  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+child);    // child entry
    if (regKey->HasValue("UIDummyChild")) {                // if there is a child then delete this first
      regKey->QueryValue("UIDummyChild", regData);
      dChild(regData, child);
    }  

    regKey=new wxRegKey(driverPos+"\\"+currentTab);               // game reg folder
    regKey->DeleteKey(child);                                    // delete reg keys
    if (currentTab=="D3D") {
      regKey=new wxRegKey(driverPos+"\\DDraw");              // game reg folder for DDraw
      if (regKey->HasSubKey(child)) regKey->DeleteKey(child);
    }  

    regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+parent);  // parent game entry
    regKey->DeleteValue("UIDummyChild");             // delete child association                                   
  }    
  list.Remove(child);                           // delete the file entry from the list       
  redrawChoiceList(parent);  // re-read the list into the drop down field
};

void BasicFrame::delEntry (wxCommandEvent & event) {         // game profile should be deleted

  if (wxMessageBox( "Deleting this entry could not be restored via reset. All children will be deleted too without further notification.\nDo you really want to delete it?",
                    "Question: Delete entry?", wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION )==wxYES) {
                               
    if (childFileLabel->GetValue()!="(None)")         // if there is a child then delete this first
      dChild(childFileLabel->GetValue(), dropDown->GetStringSelection());
    
    if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
      regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());      // current game entry
      if (regKey->HasValue("UIDummyParent")) {
        regKey->QueryValue("UIDummyParent", regData);
        regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+regData);        // parent game entry
        regKey->DeleteValue("UIDummyChild");             // delete child association                                   
      }                                           

      regKey=new wxRegKey(driverPos+"\\"+currentTab);            // game reg folder
      regKey->DeleteKey(dropDown->GetStringSelection());         // delete reg keys
      if (currentTab=="D3D") {
        regKey=new wxRegKey(driverPos+"\\DDraw");              // game reg folder for DDraw
        if (regKey->HasSubKey(dropDown->GetStringSelection())) regKey->DeleteKey(dropDown->GetStringSelection());
      }  
    }

    if (list.GetCount()>1) {        // if there is more then one entry...
      list.Remove(dropDown->GetSelection());    // ...remove the current entry from the list
      if ((unsigned)dropDown->GetSelection()==list.GetCount())   // if the last entry should be deleted...
        redrawChoiceList(dropDown->GetSelection()-1);            // ...set the drop down selection to the second last entry
      else redrawChoiceList(dropDown->GetSelection());  // ... else leave it at the current position
    }
    else {                                     // there is only one entry (which should be deleted)
      list.Empty();                            // clear the list
      redrawChoiceList(0);                     // ...set the drop down selection to the second last entry
    }     
  }  
};

void BasicFrame::addChild (wxCommandEvent & event) {        // add child

  wxString tempString;                  // temporary variables
  wxFileDialog* openFileDialog = new wxFileDialog (this, "Choose a file", "", "", "*.*", wxOPEN | wxCHANGE_DIR );    // file dialog for opening files
  if (openFileDialog->ShowModal() == wxID_OK) {             // file has been chosen
  
    if (currentTab=="OpenGL")
      tempString=openFileDialog->GetFilename();                 // OpenGL reg settings recognizes full file name
    else tempString=(openFileDialog->GetFilename()).Left(15);   // D3D reg settings recognizes onyl first 15 letters of file name

    if (list.Index(tempString, FALSE)==wxNOT_FOUND) {                         // there is no entry with the same filename  

      if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
        regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+tempString);         // new game reg folder 
        if (regKey->Create()) {                                                 // create reg key
          regKey->SetValue("UIDummyPath", openFileDialog->GetDirectory());      // create path entry
          regKey->SetValue("UIDummyParent", dropDown->GetStringSelection());    // add parent file
          regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());   // parent reg key
          regKey->SetValue("UIDummyChild", tempString);                                         // add child file
         
         // add parent reg values to the child
          regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+tempString);         // child reg key 
          if (currentTab=="D3D") {                            // Direct3D
            for (int i=0; i<38; i++) {
              switch (i) {
                case ID_DISABLE_XSS:                          // deactivate hor. antialias (standard 1)
                case ID_DISABLE_YSS:                          // deavtivate vert. antialias (standard 1)
                case ID_DISABLE_ENTNL:                        // deactivate Enhanced TnL (standard 1) 
                    if (!RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "0");
                    else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
                    break;     
                default:
                    if (RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "1");
                    else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
                    break;    
              }           
            }
            if (otherRegBoxDDraw->IsChecked()) {
              regKey=new wxRegKey(driverPos+"\\DDraw\\"+tempString);        // game reg folder for DDraw
              if (!regKey->Exists()) regKey->Create();
            } 
          }  
          else {                   // OpenGL
            for (int i=38; i<64; i++) {
              switch (i) {
                case ID_ENABLE_DXT1:                         // activate DXT1 compression (standard 1)
                case ID_DEPTH_BUFFERLESS_HSR:                // activate depth buffer (standard 1)
                case ID_DEPTH_STORE_DEFAULT:                 // store depth-buffer (standard 1)
                case ID_SYNC_DEPTHBUFFER:                    // activate asynch. depth buffer (standard 1)
                case ID_STENCIL_BUFFERLESS_SB:               // activate stencil buffer (standard 1)
                case ID_STENCIL_STORE_DEFAULT:               // store stencil-buffer (standard 1)
                case ID_SYNC_STENCILBUFFER:                  // activate asynch. stencil  buffer (standard 1)
                case ID_ENABLE_VSYNC:                        // activate vert. sync. (standard 1)
                case ID_COMPRESS_LIGHTMAP:                  // (1)
                    if (!RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "0");
                    else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
                    break;    
                default:
                    if (RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "1");
                    else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
                    break;     
              }           
            }           
          }

          list.Add(tempString);   // add file to the list
          list.Sort(CompareStringNocase);                            // sort list alphabetically   
          redrawChoiceList(dropDown->GetSelection());           // put the list entries in the drop down field   
        }                                                                                  
      }
      else {
        list.Add(tempString);       // add file to the list
        list.Sort(CompareStringNocase);                            // sort list alphabetically   
        redrawChoiceList(dropDown->GetSelection());           // put the list entries in the drop down field   
      }                                                                                  
    }  
    else {
      wxMessageBox( "There is already a profil with this filename.\nPlease choose another file.", "Error: Duplicate Filename", wxOK | wxICON_ERROR  );
    }
  }
};

void BasicFrame::freeChild (wxCommandEvent & event) {    // freeing a child (reg entry stays alive)

  if (wxMessageBox( "Freeing this child could not be restored via reset.\nDo you really want to do it?",
                    "Question: Free child?", wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION )==wxYES) {

    if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
      regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+childFileLabel->GetValue());
      regKey->DeleteValue("UIDummyParent");             // delete child association                                   
      regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());
      regKey->DeleteValue("UIDummyChild");              // delete parent association                                 
    }
    redrawChoice(dropDown->GetSelection());    // redraw current settings
  }
}

void BasicFrame::delChild (wxCommandEvent & event) {   // deleting a child

  if (wxMessageBox( "Deleting this child could not be restored via reset. All children of this child will be deleted too without further notification.\nDo you really want to delete it?",
                    "Question: Delete child?", wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION )==wxYES) {
    dChild(childFileLabel->GetValue(), dropDown->GetStringSelection());
  }
};

void BasicFrame::resetEntry (wxCommandEvent & event) {  // reset the current settings
  redrawChoice(dropDown->GetSelection());
};

void BasicFrame::aChanges (wxString game) {          // add reg keys recursively

  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+game);    // game entry
    if (regKey->HasValue("UIDummyChild")) {            // if there is a child then add reg keys first there
      regKey->QueryValue("UIDummyChild", regData);
      aChanges(regData);
    }
    regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+game);         // game reg folder
    regKey->SetValue("UIDummyName", gameLabel->GetValue());           // set game name

    if (currentTab=="D3D") {                            // Direct3D
      for (int i=0; i<38; i++) {
        switch (i) {
          case ID_DISABLE_XSS:                          // deactivate hor. antialias (standard 1)
          case ID_DISABLE_YSS:                          // deavtivate vert. antialias (standard 1)
          case ID_DISABLE_ENTNL:                        // deactivate Enhanced TnL (standard 1) 
              if (!RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "0");
              else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
              break;     
          default:
              if (RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "1");
              else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
              break;    
        }           
      }
      if (otherRegBoxDDraw->IsChecked()) {
        regKey=new wxRegKey(driverPos+"\\DDraw\\"+game);        // game reg folder for DDraw
        if (!regKey->Exists()) regKey->Create();
      }
      else {
        regKey=new wxRegKey(driverPos+"\\DDraw");                // game folder for DDraw
        if (regKey->HasSubKey(game)) regKey->DeleteKey(game);
      }  
    }  
    else {                   // OpenGL
      for (int i=38; i<64; i++) {
        switch (i) {
          case ID_ENABLE_DXT1:                         // activate DXT1 compression (standard 1)
          case ID_DEPTH_BUFFERLESS_HSR:                // activate depth buffer (standard 1)
          case ID_DEPTH_STORE_DEFAULT:                 // store depth-buffer (standard 1)
          case ID_SYNC_DEPTHBUFFER:                    // activate asynch. depth buffer (standard 1)
          case ID_STENCIL_BUFFERLESS_SB:               // activate stencil buffer (standard 1)
          case ID_STENCIL_STORE_DEFAULT:               // store stencil-buffer (standard 1)
          case ID_SYNC_STENCILBUFFER:                  // activate asynch. stencil  buffer (standard 1)
          case ID_ENABLE_VSYNC:                        // activate vert. sync. (standard 1)
          case ID_COMPRESS_LIGHTMAP:                   // (1)
              if (!RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "0");
              else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
              break;    
          default:
              if (RegBox[i]->GetValue()) regKey->SetValue(RegSet[i], "1");
              else if (regKey->HasValue(RegSet[i])) regKey->DeleteValue(RegSet[i]);
              break;     
        }           
      }           
    }
  }    
};

void BasicFrame::applyChanges (wxCommandEvent & event) { // apply settings and write it to the registry

  aChanges(dropDown->GetStringSelection());           // apply reg settings to whole family
  redrawChoice(dropDown->GetSelection());         // redraw current settings
};

void BasicFrame::addLastEntry( wxCommandEvent & event) {         // add last used application
  
  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    regKey=new wxRegKey(driverPos+"\\"+lastGameType+"\\"+lastApp);       // new game reg folder 
    if (regKey->Create()) {                                       // create reg key
      if (lastGameType=="D3D") {
        regKey=new wxRegKey(driverPos+"\\DDraw\\"+lastApp);        // game reg folder for DDraw
        if (!regKey->Exists()) regKey->Create();
      }  
      list.Add(lastApp);                                    // add file to the list
      list.Sort(CompareStringNocase);                      // sort list alphabetically   
      gotoLastEntry(event);                                // got last app
    } 
  }
  else {
    list.Add(lastApp);                                    // add file to the list
    list.Sort(CompareStringNocase);                      // sort list alphabetically   
    redrawChoiceList(lastApp);                      // put the list entries in the drop down field   
  }           
};

void BasicFrame::gotoLastEntry( wxCommandEvent & event) {          // goto last used application
  if (layout->GetPageText(layout->GetSelection())!=lastGameType) {   // we are already in the right tab
    if (lastGameType=="D3D" ) layout->SetSelection(0);             // change to D3D tab                           
    else if (lastGameType=="OpenGL") layout->SetSelection(1);     // change to OGL tab
  }  
  redrawChoiceList(lastApp);                                        // redraw current settings
};

void BasicFrame::rereadReg (wxCommandEvent & event) {     // rereads the registry settings into the list
  long tempint;                                        // tempory long integer for reading values
  regKey=new wxRegKey(driverPos+"\\D3D");             // D3D reg folder                                                         
  listD3D.Empty();
  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    if (regKey->GetFirstKey(regData, tempint)) {         // get first profile name
      do listD3D.Add(regData);                                // add filenames...
      while (regKey->GetNextKey(regData, tempint));        // ...until there is no more
    }
  }    
  listD3D.Sort(CompareStringNocase);                           // sort list
  regKey=new wxRegKey(driverPos+"\\OpenGL");                 // OGL reg folder                                                         
  listOGL.Empty();
  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    if (regKey->GetFirstKey(regData, tempint)) {         // get first profile name
      do listOGL.Add(regData);                                // add filenames...
      while (regKey->GetNextKey(regData, tempint));        // ...until there is no more
    }
  }    
  listOGL.Sort(CompareStringNocase);                           // sort list

  if (layout->GetPageText(layout->GetSelection())=="OpenGL") {
    currentTab="OpenGL";
    list=listOGL;
  }
  else if (layout->GetPageText(layout->GetSelection())=="Direct3D") {
    currentTab="D3D";
    list=listD3D;
  }  
  redrawChoiceList(0);                      // re-read the list into the drop down field
};

void BasicFrame::loadReg (wxCommandEvent & event) {     // load registry settings from file

  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode

    wxFileDialog* openFileDialog = new wxFileDialog (this, "Load from...", "", "backup"+version+".reg", "Registry files (*.reg)|*.reg", wxOPEN | wxCHANGE_DIR );    // file dialog for opening files
    if (openFileDialog->ShowModal() == wxID_OK) {             // file has been chosen
      wxString tempString="REGEDIT /S "+openFileDialog->GetFilename();
      system (tempString);
    }
  }
  rereadReg(event);       // reread the registry into the list
};

void BasicFrame::saveReg (wxCommandEvent & event) {     // save registry settings from file

  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode

    wxFileDialog* saveFileDialog = new wxFileDialog (this, "Save to...", "", "backup"+version+".reg", "Registry files (*.reg)|*.reg", wxSAVE | wxOVERWRITE_PROMPT | wxCHANGE_DIR );    // file dialog for saving files
    if (saveFileDialog->ShowModal() == wxID_OK) {             // file has been chosen
      wxString tempString="REGEDIT /E "+saveFileDialog->GetFilename()+" \""+driverPos+"\"";
      system (tempString);
    }
  }    
};

void BasicFrame::toggleDemoMode (wxCommandEvent & event) {   // toggles demo mode
  if (demoMode) optionMenu->Check(ID_DEMO_MODE, true);  // demo mode for non-kyro cards cannot be toggled
  else {
    if (optionMenu->IsChecked(ID_DEMO_MODE)) SetTitle(name+" version "+version+" (Demo Mode)");
    else SetTitle(name+" version "+version);                 // set new frame title
    wxCommandEvent event;
    rereadReg(event);            // repaint list and dropdown field
  }
};

void BasicFrame::selectPath (wxCommandEvent & event) {     // select a path where the game is located
  if (event.GetId()==ID_PATH_BOX) {         // Path Box was clicked
    if (pathBox->IsChecked()) {              // new path should be declared
      wxDirDialog* openDirDialog = new wxDirDialog(this);    // open directory
      if (openDirDialog->ShowModal() == wxID_OK) {             // directory has been chosen
        if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
          regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());  // game reg folder
          regKey->SetValue("UIDummyPath", openDirDialog->GetPath());        // create path entry
        }  
      }
    }
    else {                //  old path should be deleted            
     regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());  // game reg folder
     if (regKey->HasValue("UIDummyPath")) regKey->DeleteValue("UIDummyPath");      // delete path entry                                   
    }
  }
  redrawChoice(dropDown->GetSelection());         // redraw current settings
};

void BasicFrame::quitApp (wxCommandEvent & event) {   // quit application

  if (!entryChanged || wxMessageBox( "You made some changes that are not saved yet.\nDo you really want to quit and loose data?",
                    "Question: Really quit?", wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION )==wxYES) {
    Close();        // close frame
  }  
};

void BasicFrame::about (wxCommandEvent & event) {   // show about dialog
  wxMessageBox( name+" version "+version+"\n\nby Dominik Wagenfuehr (dominik@cyskat.de)\n\nCreated: 25. March 2005\nModified: "+datum,
                    "About", wxOK);
};

void BasicFrame::help (wxCommandEvent & event) {   // show help dialog
  if (!wxFile::Exists("help\\index.htm")) {
      wxMessageBox( "The help file ist not located at "+wxGetCwd()+"\\help\\index.htm!\nPlease download the help file and unzip it to the right directory.", "Error: Help file not found.", wxOK | wxICON_ERROR  );
  }    
  else {
    wxFrame *helpFrame = new wxFrame(this, wxID_ANY, "KCP Help");
    wxHtmlWindow * htmlwin = new wxHtmlWindow(helpFrame);
    htmlwin->LoadPage("help\\index.htm");
    helpFrame->Show();
  }
};

void BasicFrame::startApp( wxCommandEvent & event) {         // start application
  wxString tempString;
  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());  // game reg folder
    regKey->QueryValue("UIDummyPath", regData);                        // query path                                 
    tempString=(dropDown->GetStringSelection()).BeforeFirst('.');
    if (tempString=="game" || tempString=="kart" || tempString=="lithtech" || tempString=="main" || tempString=="setup")
      regData+="\\"+tempString+".exe";      // add filename
    else regData+="\\"+dropDown->GetStringSelection();      // add filename
    if (!wxFile::Exists(regData)) {
      wxMessageBox( "The application you want to start does not exist with this path and filename. Please check your settings!", "Error: Application is missing.", wxOK | wxICON_ERROR  );
    }
    else {
      regData="start \"\" \""+regData+"\"";
      system(regData);                                                 // start application
    }  
  }  
};

void BasicFrame::gameChanged( wxCommandEvent & event) {         // game name has been changed
  if (!demoMode && !optionMenu->IsChecked(ID_DEMO_MODE)) {   // only access registry in non demo mode
    regKey=new wxRegKey(driverPos+"\\"+currentTab+"\\"+dropDown->GetStringSelection());  // game reg folder
    if (regKey->HasValue("UIDummyName")) {                      // game had a name and is changed 
      regKey->QueryValue("UIDummyName", regData);                             // query path 
      if (gameLabel->GetValue()!="" && regData!=gameLabel->GetValue()) { 
        entryChanged=true;            // there has something changed
        applyButton->Enable();        // ... so the apply button and ...
        resetButton->Enable();        // ... the reset button are enabled
      }
    }
    else {                                   // game had no name until now
      if (gameLabel->GetValue()!="") {
        entryChanged=true;            // there has something changed
        applyButton->Enable();        // ... so the apply button and ...
        resetButton->Enable();        // ... the reset button are enabled
      }         
    }  
  }  
};

void BasicFrame::checkBoxChange (wxCommandEvent & event) {      // some checkbox has been clicked
  if (event.GetId()!=ID_OTHER_D3D && event.GetId()!=ID_OTHER_OGL) {
    entryChanged=true;            // there has something changed
    applyButton->Enable();        // ... so the apply button and ...
    resetButton->Enable();        // ... the reset button are enabled

    // message(RegSet[event.GetId()]);  // internal use only

    if (event.GetId()==ID_ENABLE_SCENESS) {    //activate antialias (standard 0)
      RegBox[ID_DISABLE_XSS]->Enable(RegBox[ID_ENABLE_SCENESS]->GetValue());
      RegBox[ID_DISABLE_YSS]->Enable(RegBox[ID_ENABLE_SCENESS]->GetValue());
    } 
  }
  else {
    otherRegBoxD3D->SetValue(!otherRegBoxD3D->GetValue());
    otherRegBoxOGL->SetValue(!otherRegBoxOGL->GetValue());
  }  
};

void BasicApplication::SetConsts(void) {
  // Direct3D Settings (order relevant)
  RegSet.Add("ForceLargeTexToDXT1");
  RegSet.Add("UseRHW");
  RegSet.Add("ZLoad"); 
  RegSet.Add("ZStore"); 
  RegSet.Add("ZTranslate");

  RegSet.Add("EnableSceneSS");
  RegSet.Add("DisableXSS");
  RegSet.Add("DisableYSS"); 
  RegSet.Add("EnableSoftPT");
  RegSet.Add("EnableTextureSS");
  RegSet.Add("DisableAnisotropicFiltering");
  RegSet.Add("EnableTextureTLF");
  RegSet.Add("DisableFastTLF");
  RegSet.Add("DoNotDisableFastTLF");

  RegSet.Add("DisableEnTnL");
  RegSet.Add("DisableMidSceneRender");
  RegSet.Add("DXTExposeCaps");
  RegSet.Add("ExposeCubeMapCaps");
  RegSet.Add("Expose2kTextures");
  RegSet.Add("NoRendersTillFlip");
  RegSet.Add("PretendStencilBufferIs8Bit");
  RegSet.Add("ISPHalfOffset");
  RegSet.Add("TAHalfOffset");

  RegSet.Add("EnablePowerVRLogo");
  RegSet.Add("NoVSYNCLock");

  RegSet.Add("Disable24Bit3DDevice");
  RegSet.Add("DisableAlphaTest");
  RegSet.Add("DisableCopyBlt3D");
  RegSet.Add("DisableDXT1");
  RegSet.Add("DisableHardwareCursor");
  RegSet.Add("DisableMipMapping");
  RegSet.Add("DisableRenderOnLock");
  RegSet.Add("DisableStencil");
  RegSet.Add("DoNotWaitOnLock");
  RegSet.Add("EnableComplexSceneMode");
  RegSet.Add("EnableTextureManagement");
  RegSet.Add("OnlyUseLastFogColour");
  RegSet.Add("ReloadOnSubblt");

  // OpenGL settings (order relevant)
  RegSet.Add("ComplexSceneMode");
  RegSet.Add("EnableFastColor");
  RegSet.Add("ForceSingleBuffered");
  RegSet.Add("HonourDoubleBuffer");
  RegSet.Add("IgnoreScissor");

  RegSet.Add("EnableDXT1");
  RegSet.Add("DisableAnisotropicTexFilter");
  RegSet.Add("ForceAnisotropicTexFilter");
  RegSet.Add("SceneAntiAliasX");
  RegSet.Add("SceneAntiAliasY");

  RegSet.Add("Compress1BitAlphaTextures");
  RegSet.Add("CompressDXT35Textures");
  RegSet.Add("CompressLightMap");
  RegSet.Add("CompressRGBTextures");

  RegSet.Add("DepthBufferlessHSR");
  RegSet.Add("DepthStoreDefault");
  RegSet.Add("SynchronousDepthBuffer");
  RegSet.Add("StencilBufferlessSB");
  RegSet.Add("StencilStoreDefault");
  RegSet.Add("SynchronousStencilBuffer");

  RegSet.Add("EnablePowerVRLogo");
  RegSet.Add("EnableVsync");

  RegSet.Add("AllowExclusiveMode");
  RegSet.Add("CheckForWindowChanges");
  RegSet.Add("DisableAlphaTest");
  RegSet.Add("HandleZTrick");
};
   
bool BasicApplication::OnInit() {                                // init frame
  SetConsts();
  BasicFrame *frame = new BasicFrame(name+" version "+version,
              wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION, 602,600); //556
  frame->Show(TRUE);
  SetTopWindow(frame);
  return (TRUE);	
};
